package nianioTest.nianio.cmdsfact;

import nianio.AppCommand;
import nianio.AppLogicCmd;
import nianio.Event;
import nianio.SelectFileCmd;
import nianio.complib.CompLibIdEvent;
import nianio.complib.DialogIdEvent;
import nianio.complib.FileChooserIdEvent;
import nianio.complib.FrameEvent;
import nianio.complib.FrameIdEvent;
import nianio.complib.MessageDialogIdEvent;
import nianio.complib.controls.ButtonEvent;
import nianio.complib.controls.ButtonIdEvent;
import nianio.complib.controls.ControlIdEvent;
import nianio.complib.controls.RadioButtonEvent;
import nianio.complib.controls.RadioButtonGroupEvent;
import nianio.complib.controls.RadioButtonGroupIdEvent;
import nianio.complib.controls.RadioButtonIdEvent;
import nianio.complib.controls.TextBoxEvent;
import nianio.complib.controls.TextBoxIdEvent;
import nianio.complib.messagedialog.MessageDialogEvent;
import nianio.genrap.GenrapPluginEvent;
import nianio.ids.ButtonId;
import nianio.ids.FileChooserId;
import nianio.ids.FileInputId;
import nianio.ids.FrameId;
import nianio.ids.MessageDialogId;
import nianio.ids.RadioButtonGroupId;
import nianio.ids.RadioButtonId;
import nianio.ids.TextBoxId;
import ocean.common.Err;

public class AppLogicCmdFact {

	public static AppLogicCmd tryCreate(AppCommand cmd) {
		if (cmd.isRun()) {
			return AppLogicCmd.createRun();
		} else if (cmd.isEvent()) {
			return tryCreate(cmd.asEvent());
		} else
			throw Err.errM(cmd);
	}

	private static AppLogicCmd tryCreate(Event event) {
		if (event.isComplib()) {
			return tryCreate(event.asComplib());
		} else if (event.isGenrap()) {
			return create(event.asGenrap());
		} else
			throw Err.errM(event);
	}

	private static AppLogicCmd create(GenrapPluginEvent event) {
		return AppLogicCmd.createReportGenerated(event);
	}

	private static AppLogicCmd tryCreate(CompLibIdEvent event) {
		if (event.isControl()) {
			return tryCreate(event.asControl());
		} else if (event.isDialog()) {
			return tryCreate(event.asDialog());
		} else if (event.isFrame()) {
			return tryCreate(event.asFrame());
		} else
			throw Err.errM(event);
	}

	private static AppLogicCmd tryCreate(FrameIdEvent event) {
		FrameId r = event.getId();
		if (r.isMainFrame()) {
			return tryCreateFromMainFrame(event.getEvent());
		} else
			throw Err.errM(r);
	}

	private static AppLogicCmd tryCreateFromMainFrame(FrameEvent event) {
		if (event.isClosRequest()) {
			return AppLogicCmd.createClose();
		} else
			throw Err.errM(event);
	}

	private static AppLogicCmd tryCreate(DialogIdEvent event) {
		if (event.isMessageDialog()) {
			return create(event.asMessageDialog());
		} else if (event.isFileChooser()) {
			return create(event.asFileChooser());
		} else
			throw Err.errM(event);
	}

	public static AppLogicCmd create(MessageDialogIdEvent event) {
		MessageDialogId r = event.getId();
		if (r.isGenerateFinish()) {
			return create(event.getEvent());
		} else
			throw Err.errM(r);
	}

	private static AppLogicCmd create(MessageDialogEvent event) {
		if (event.isClosed()) {
			return create(event.asClosed());
		} else
			throw Err.errM(event);
	}

	private static AppLogicCmd create(MessageDialogId id) {
		if (id.isGenerateFinish()) {
			return AppLogicCmd.createMessageDialogClosed(id.asGenerateFinish());
		} else
			throw Err.errM(id);
	}

	public static AppLogicCmd create(FileChooserIdEvent event) {
		SelectFileCmd value = SelectFileCmdFact.create(event.getEvent());
		FileChooserId r = event.getId();
		if (r.isReport()) {
			return AppLogicCmd.createReportFileCmd(value);
		} else if (r.isData()) {
			return AppLogicCmd.createXmlFileCmd(value);
		} else if (r.isSchema()) {
			return AppLogicCmd.createXsdFileCmd(value);
		} else if (r.isOutput()) {
			return AppLogicCmd.createOutputFileCmd(value);
		} else
			throw Err.errM(r);
	}

	private static AppLogicCmd tryCreate(ControlIdEvent event) {
		if (event.isButtonEvent()) {
			return create(event.asButtonEvent());
		} else if (event.isRadioButtonEvent()) {
			return create(event.asRadioButtonEvent());
		} else if (event.isTextBoxEvent()) {
			return tryCreate(event.asTextBoxEvent());
		} else if (event.isRadioButtonGroupIdEvent()) {
			return tryCreate(event.asRadioButtonGroupIdEvent());
		} else
			throw Err.errM(event);
	}

	public static AppLogicCmd tryCreate(RadioButtonGroupIdEvent event) {
		RadioButtonGroupId id = event.getId();
		if (id.isOutputType()) {
			return create(event.getCmd());
		} else
			throw Err.errM(id);
	}

	private static AppLogicCmd create(RadioButtonGroupEvent cmd) {
		if (cmd.isSelected()) {
			RadioButtonId r = cmd.asSelected();
			if (r.isPdfOutput()) {
				return AppLogicCmd.createSetPdfOutput();
			} else if (r.isHtmlOutput()) {
				return AppLogicCmd.createSetHtmlOutput();
			} else if (r.isXlsOutput()) {
				return AppLogicCmd.createSetXlsOutput();
			} else
				throw Err.errM(r);
		} else
			throw Err.errM(cmd);
	}

	public static AppLogicCmd create(RadioButtonIdEvent event) {
		RadioButtonId r = event.getId();
		RadioButtonEvent cmd = event.getCmd();
		if (r.isPdfOutput()) {
			return createPdfOutput(cmd);
		} else if (r.isHtmlOutput()) {
			return createHtmlOutput(cmd);
		} else if (r.isXlsOutput()) {
			return createXlsOutput(cmd);
		} else
			throw Err.errM(r);
	}

	private static AppLogicCmd createPdfOutput(RadioButtonEvent cmd) {
		if (cmd.isSelected()) {
			return AppLogicCmd.createSetPdfOutput();
		} else
			throw Err.errM(cmd);
	}

	private static AppLogicCmd createHtmlOutput(RadioButtonEvent cmd) {
		if (cmd.isSelected()) {
			return AppLogicCmd.createSetHtmlOutput();
		} else
			throw Err.errM(cmd);
	}

	private static AppLogicCmd createXlsOutput(RadioButtonEvent cmd) {
		if (cmd.isSelected()) {
			return AppLogicCmd.createSetXlsOutput();
		} else
			throw Err.errM(cmd);
	}

	public static AppLogicCmd create(ButtonIdEvent event) {
		ButtonId r = event.getId();
		if (r.isFileInput()) {
			return create(r.asFileInput(), event.getCmd());
		} else if (r.isGenerate()) {
			return createGenerate(event.getCmd());
		} else
			throw Err.errM(r);
	}

	private static AppLogicCmd create(FileInputId id, ButtonEvent cmd) {
		if (id.isRaport()) {
			return AppLogicCmd.createReportFileCmd(SelectFileCmdFact.create(cmd));
		} else if (id.isData()) {
			return AppLogicCmd.createXmlFileCmd(SelectFileCmdFact.create(cmd));
		} else if (id.isSchema()) {
			return AppLogicCmd.createXsdFileCmd(SelectFileCmdFact.create(cmd));
		} else if (id.isOutput()) {
			return AppLogicCmd.createOutputFileCmd(SelectFileCmdFact.create(cmd));
		} else
			throw Err.errM(id);
	}

	private static AppLogicCmd createGenerate(ButtonEvent cmd) {
		if (cmd.isFired()) {
			return AppLogicCmd.createGenerateRequest();
		} else
			throw Err.errM(cmd);
	}

	public static AppLogicCmd tryCreate(TextBoxIdEvent event) {
		TextBoxId id = event.getId();
		if (id.isFileInput()) {
			return tryCreate(id.asFileInput(), event.getCmd());
		} else
			throw Err.errM(id);
	}

	private static AppLogicCmd tryCreate(FileInputId id, TextBoxEvent cmd) {
		SelectFileCmd c = SelectFileCmdFact.tryCreate(cmd);
		if (c == null)
			return null;
		if (id.isRaport()) {
			return AppLogicCmd.createReportFileCmd(c);
		} else if (id.isData()) {
			return AppLogicCmd.createXmlFileCmd(c);
		} else if (id.isSchema()) {
			return AppLogicCmd.createXsdFileCmd(c);
		} else if (id.isOutput()) {
			return AppLogicCmd.createOutputFileCmd(c);
		} else
			throw Err.errM(id);
	}
}
