package nianio.complib;

import schema.runtime.datatypes.ImmObject;
import nianio.complib.controls.ControlCommand;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class CompLibCommand = variant {
 *	    control of ControlCommand  
 *	 |  dialog of DialogCommand  
 *	 |  frame of FrameCommand  
 * }</pre>
 */ 
public class CompLibCommand implements ImmObject {
	private static final int TYPE_HASH = -1468140139;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private CompLibCommand(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static CompLibCommand createControl(ControlCommand value) {
		return new CompLibCommand(0, value);
	}

	public static CompLibCommand createDialog(DialogCommand value) {
		return new CompLibCommand(1, value);
	}

	public static CompLibCommand createFrame(FrameCommand value) {
		return new CompLibCommand(2, value);
	}



	/** @return true, when this is control */
	public boolean isControl() {
		return variant == 0;
	}

	/** @return variant value, when this is control; throws Exception otherwise */
	public ControlCommand asControl() {
		SchemaException.ensure(isControl());
		return (ControlCommand)value;
	}

	/** @return true, when this is dialog */
	public boolean isDialog() {
		return variant == 1;
	}

	/** @return variant value, when this is dialog; throws Exception otherwise */
	public DialogCommand asDialog() {
		SchemaException.ensure(isDialog());
		return (DialogCommand)value;
	}

	/** @return true, when this is frame */
	public boolean isFrame() {
		return variant == 2;
	}

	/** @return variant value, when this is frame; throws Exception otherwise */
	public FrameCommand asFrame() {
		SchemaException.ensure(isFrame());
		return (FrameCommand)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isControl()) {
			;
		} else if (isDialog()) {
			;
		} else if (isFrame()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isControl()) {
			asControl();
		} else if (isDialog()) {
			asDialog();
		} else if (isFrame()) {
			asFrame();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isControl())
			return "control" + "(" + asControl() + ")";
		if (isDialog())
			return "dialog" + "(" + asDialog() + ")";
		if (isFrame())
			return "frame" + "(" + asFrame() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(CompLibCommand obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof CompLibCommand) 
			&& this.eq((CompLibCommand)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for CompLibCommand
	 */
	public static interface MM {
		public void invoke(CompLibCommand cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for CompLibCommand
	 */
	public static interface MMIntf {
		public void control(ControlCommand value);
		public void dialog(DialogCommand value);
		public void frame(FrameCommand value);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for CompLibCommand
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(CompLibCommand cmd) {
				 if (cmd.isControl()) {
					mm.control(cmd.asControl());
				} else if (cmd.isDialog()) {
					mm.dialog(cmd.asDialog());
				} else if (cmd.isFrame()) {
					mm.frame(cmd.asFrame());
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void control(ControlCommand value) {
				mm.invoke(CompLibCommand.createControl(value));
			}
		
			public void dialog(DialogCommand value) {
				mm.invoke(CompLibCommand.createDialog(value));
			}
		
			public void frame(FrameCommand value) {
				mm.invoke(CompLibCommand.createFrame(value));
			}
		
		}
	}
}