package nianio.complib;

import nianio.ids.FileChooserId;
import nianio.complib.filechooser.FileChooserEvent;
import nianio.ids.UserDialogId;
import nianio.complib.messagedialog.MessageDialogEvent;
import nianio.ids.MessageDialogId;
import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class DialogIdEvent = variant {
 *	    messageDialog of MessageDialogIdEvent  
 *	 |  fileChooser of FileChooserIdEvent  
 *	 |  userDialog of UserDialogIdEvent  
 * }</pre>
 */ 
public class DialogIdEvent implements ImmObject {
	private static final int TYPE_HASH = -706594889;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private DialogIdEvent(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static DialogIdEvent createMessageDialog(MessageDialogIdEvent value) {
		return new DialogIdEvent(0, value);
	}

	public static DialogIdEvent createFileChooser(FileChooserIdEvent value) {
		return new DialogIdEvent(1, value);
	}

	public static DialogIdEvent createUserDialog(UserDialogIdEvent value) {
		return new DialogIdEvent(2, value);
	}


	public static DialogIdEvent mkMessageDialog(
			MessageDialogId id
			, MessageDialogEvent event
	) {
		return createMessageDialog( MessageDialogIdEvent.create(
							id
							, event
							 ));
	}
	
	public static DialogIdEvent mkFileChooser(
			FileChooserId id
			, FileChooserEvent event
	) {
		return createFileChooser( FileChooserIdEvent.create(
							id
							, event
							 ));
	}
	
	public static DialogIdEvent mkUserDialog(
			UserDialogId id
			, UserDialogEvent event
	) {
		return createUserDialog( UserDialogIdEvent.create(
							id
							, event
							 ));
	}
	

	/** @return true, when this is messageDialog */
	public boolean isMessageDialog() {
		return variant == 0;
	}

	/** @return variant value, when this is messageDialog; throws Exception otherwise */
	public MessageDialogIdEvent asMessageDialog() {
		SchemaException.ensure(isMessageDialog());
		return (MessageDialogIdEvent)value;
	}

	/** @return true, when this is fileChooser */
	public boolean isFileChooser() {
		return variant == 1;
	}

	/** @return variant value, when this is fileChooser; throws Exception otherwise */
	public FileChooserIdEvent asFileChooser() {
		SchemaException.ensure(isFileChooser());
		return (FileChooserIdEvent)value;
	}

	/** @return true, when this is userDialog */
	public boolean isUserDialog() {
		return variant == 2;
	}

	/** @return variant value, when this is userDialog; throws Exception otherwise */
	public UserDialogIdEvent asUserDialog() {
		SchemaException.ensure(isUserDialog());
		return (UserDialogIdEvent)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isMessageDialog()) {
			;
		} else if (isFileChooser()) {
			;
		} else if (isUserDialog()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isMessageDialog()) {
			asMessageDialog();
		} else if (isFileChooser()) {
			asFileChooser();
		} else if (isUserDialog()) {
			asUserDialog();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isMessageDialog())
			return "messageDialog" + "(" + asMessageDialog() + ")";
		if (isFileChooser())
			return "fileChooser" + "(" + asFileChooser() + ")";
		if (isUserDialog())
			return "userDialog" + "(" + asUserDialog() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(DialogIdEvent obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof DialogIdEvent) 
			&& this.eq((DialogIdEvent)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for DialogIdEvent
	 */
	public static interface MM {
		public void invoke(DialogIdEvent cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for DialogIdEvent
	 */
	public static interface MMIntf {
		public void messageDialog(
				 MessageDialogId id
				, MessageDialogEvent event
		);
		public void fileChooser(
				 FileChooserId id
				, FileChooserEvent event
		);
		public void userDialog(
				 UserDialogId id
				, UserDialogEvent event
		);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for DialogIdEvent
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(DialogIdEvent cmd) {
				 if (cmd.isMessageDialog()) {
					MessageDialogIdEvent val = cmd.asMessageDialog();
					mm.messageDialog(
							 val.getId()
							, val.getEvent()
					);
				} else if (cmd.isFileChooser()) {
					FileChooserIdEvent val = cmd.asFileChooser();
					mm.fileChooser(
							 val.getId()
							, val.getEvent()
					);
				} else if (cmd.isUserDialog()) {
					UserDialogIdEvent val = cmd.asUserDialog();
					mm.userDialog(
							 val.getId()
							, val.getEvent()
					);
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void messageDialog(
							 MessageDialogId id
							, MessageDialogEvent event
			) {
				mm.invoke( DialogIdEvent.createMessageDialog(
								MessageDialogIdEvent.create( 
									id, event)));		
			}
		
			public void fileChooser(
							 FileChooserId id
							, FileChooserEvent event
			) {
				mm.invoke( DialogIdEvent.createFileChooser(
								FileChooserIdEvent.create( 
									id, event)));		
			}
		
			public void userDialog(
							 UserDialogId id
							, UserDialogEvent event
			) {
				mm.invoke( DialogIdEvent.createUserDialog(
								UserDialogIdEvent.create( 
									id, event)));		
			}
		
		}
	}
}