package nianio.complib;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;
import nianio.ids.FrameId;


/** 
 * <pre>Autogenerated class FrameCommand = variant {
 *	    createAndShow of FrameState  
 *	 |  frame of ShownFrameIdCommand  
 * }</pre>
 */ 
public class FrameCommand implements ImmObject {
	private static final int TYPE_HASH = -1323885250;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private FrameCommand(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static FrameCommand createCreateAndShow(FrameState value) {
		return new FrameCommand(0, value);
	}

	public static FrameCommand createFrame(ShownFrameIdCommand value) {
		return new FrameCommand(1, value);
	}


	public static FrameCommand mkCreateAndShow(
			FrameId id
			, String title
			, boolean isResizable
			, CompState content
	) {
		return createCreateAndShow( FrameState.create(
							id
							, title
							, isResizable
							, content
							 ));
	}
	
	public static FrameCommand mkFrame(
			FrameId id
			, ShownFrameCommand cmd
	) {
		return createFrame( ShownFrameIdCommand.create(
							id
							, cmd
							 ));
	}
	

	/** @return true, when this is createAndShow */
	public boolean isCreateAndShow() {
		return variant == 0;
	}

	/** @return variant value, when this is createAndShow; throws Exception otherwise */
	public FrameState asCreateAndShow() {
		SchemaException.ensure(isCreateAndShow());
		return (FrameState)value;
	}

	/** @return true, when this is frame */
	public boolean isFrame() {
		return variant == 1;
	}

	/** @return variant value, when this is frame; throws Exception otherwise */
	public ShownFrameIdCommand asFrame() {
		SchemaException.ensure(isFrame());
		return (ShownFrameIdCommand)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isCreateAndShow()) {
			;
		} else if (isFrame()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isCreateAndShow()) {
			asCreateAndShow();
		} else if (isFrame()) {
			asFrame();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isCreateAndShow())
			return "createAndShow" + "(" + asCreateAndShow() + ")";
		if (isFrame())
			return "frame" + "(" + asFrame() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(FrameCommand obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof FrameCommand) 
			&& this.eq((FrameCommand)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	
}