package nianio.complib.controls;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class TextBoxCommand = variant {
 *	    enable 
 *	 |  disable 
 *	 |  setText of String  
 *	 |  setEditable of boolean  
 * }</pre>
 */ 
public class TextBoxCommand implements ImmObject {
	private static final int TYPE_HASH = -614135763;
	private int hash = 0;

	private static final TextBoxCommand CONST_ENABLE = new TextBoxCommand(0);
	private static final TextBoxCommand CONST_DISABLE = new TextBoxCommand(1);
	private final int variant;
	private final Object value;  

	private TextBoxCommand(int variant) {
		this.variant = variant;
		this.value = null;
	}

	private TextBoxCommand(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static TextBoxCommand createEnable() {
		return CONST_ENABLE;
	}

	public static TextBoxCommand createDisable() {
		return CONST_DISABLE;
	}

	public static TextBoxCommand createSetText(String value) {
		return new TextBoxCommand(2, value);
	}

	public static TextBoxCommand createSetEditable(boolean value) {
		return new TextBoxCommand(3, new Boolean(value));
	}



	/** @return true, when this is enable */
	public boolean isEnable() {
		return variant == 0;
	}


	/** @return true, when this is disable */
	public boolean isDisable() {
		return variant == 1;
	}


	/** @return true, when this is setText */
	public boolean isSetText() {
		return variant == 2;
	}

	/** @return variant value, when this is setText; throws Exception otherwise */
	public String asSetText() {
		SchemaException.ensure(isSetText());
		return ((String)value);
	}

	/** @return true, when this is setEditable */
	public boolean isSetEditable() {
		return variant == 3;
	}

	/** @return variant value, when this is setEditable; throws Exception otherwise */
	public boolean asSetEditable() {
		SchemaException.ensure(isSetEditable());
		return ((Boolean)value).booleanValue();
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isEnable()) {
			;
		} else if (isDisable()) {
			;
		} else if (isSetText()) {
			;
		} else if (isSetEditable()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isEnable()) {
			;
		} else if (isDisable()) {
			;
		} else if (isSetText()) {
			asSetText();
		} else if (isSetEditable()) {
			asSetEditable();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isEnable())
			return "enable";		
		if (isDisable())
			return "disable";		
		if (isSetText())
			return "setText" + "(" + asSetText() + ")";
		if (isSetEditable())
			return "setEditable" + "(" + asSetEditable() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(TextBoxCommand obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof TextBoxCommand) 
			&& this.eq((TextBoxCommand)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for TextBoxCommand
	 */
	public static interface MM {
		public void invoke(TextBoxCommand cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for TextBoxCommand
	 */
	public static interface MMIntf {
		public void enable();
		public void disable();
		public void setText(String value);
		public void setEditable(boolean value);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for TextBoxCommand
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(TextBoxCommand cmd) {
				 if (cmd.isEnable()) {
					mm.enable();
				} else if (cmd.isDisable()) {
					mm.disable();
				} else if (cmd.isSetText()) {
					mm.setText(cmd.asSetText());
				} else if (cmd.isSetEditable()) {
					mm.setEditable(cmd.asSetEditable());
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void enable() {
				mm.invoke(TextBoxCommand.createEnable());
			}
		
			public void disable() {
				mm.invoke(TextBoxCommand.createDisable());
			}
		
			public void setText(String value) {
				mm.invoke(TextBoxCommand.createSetText(value));
			}
		
			public void setEditable(boolean value) {
				mm.invoke(TextBoxCommand.createSetEditable(value));
			}
		
		}
	}
}