package nianio.complib.controls;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class TextBoxEvent = variant {
 *	    textChanged of String  
 *	 |  keyPressed 
 *	 |  enterPressed 
 * }</pre>
 */ 
public class TextBoxEvent implements ImmObject {
	private static final int TYPE_HASH = 2124311548;
	private int hash = 0;

	private static final TextBoxEvent CONST_KEYPRESSED = new TextBoxEvent(1);
	private static final TextBoxEvent CONST_ENTERPRESSED = new TextBoxEvent(2);
	private final int variant;
	private final Object value;  

	private TextBoxEvent(int variant) {
		this.variant = variant;
		this.value = null;
	}

	private TextBoxEvent(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static TextBoxEvent createTextChanged(String value) {
		return new TextBoxEvent(0, value);
	}

	public static TextBoxEvent createKeyPressed() {
		return CONST_KEYPRESSED;
	}

	public static TextBoxEvent createEnterPressed() {
		return CONST_ENTERPRESSED;
	}



	/** @return true, when this is textChanged */
	public boolean isTextChanged() {
		return variant == 0;
	}

	/** @return variant value, when this is textChanged; throws Exception otherwise */
	public String asTextChanged() {
		SchemaException.ensure(isTextChanged());
		return ((String)value);
	}

	/** @return true, when this is keyPressed */
	public boolean isKeyPressed() {
		return variant == 1;
	}


	/** @return true, when this is enterPressed */
	public boolean isEnterPressed() {
		return variant == 2;
	}




	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isTextChanged()) {
			;
		} else if (isKeyPressed()) {
			;
		} else if (isEnterPressed()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isTextChanged()) {
			asTextChanged();
		} else if (isKeyPressed()) {
			;
		} else if (isEnterPressed()) {
			;
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isTextChanged())
			return "textChanged" + "(" + asTextChanged() + ")";
		if (isKeyPressed())
			return "keyPressed";		
		if (isEnterPressed())
			return "enterPressed";		
		throw SchemaException.err();
	}
	
	public boolean eq(TextBoxEvent obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof TextBoxEvent) 
			&& this.eq((TextBoxEvent)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for TextBoxEvent
	 */
	public static interface MM {
		public void invoke(TextBoxEvent cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for TextBoxEvent
	 */
	public static interface MMIntf {
		public void textChanged(String value);
		public void keyPressed();
		public void enterPressed();
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for TextBoxEvent
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(TextBoxEvent cmd) {
				 if (cmd.isTextChanged()) {
					mm.textChanged(cmd.asTextChanged());
				} else if (cmd.isKeyPressed()) {
					mm.keyPressed();
				} else if (cmd.isEnterPressed()) {
					mm.enterPressed();
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void textChanged(String value) {
				mm.invoke(TextBoxEvent.createTextChanged(value));
			}
		
			public void keyPressed() {
				mm.invoke(TextBoxEvent.createKeyPressed());
			}
		
			public void enterPressed() {
				mm.invoke(TextBoxEvent.createEnterPressed());
			}
		
		}
	}
}