package nianio.genrap;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class GenrapPluginEvent = variant {
 *	    success of ReportParams  
 *	 |  failure of ReportGenerationFailure  
 * }</pre>
 */ 
public class GenrapPluginEvent implements ImmObject {
	private static final int TYPE_HASH = -477323402;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private GenrapPluginEvent(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static GenrapPluginEvent createSuccess(ReportParams value) {
		return new GenrapPluginEvent(0, value);
	}

	public static GenrapPluginEvent createFailure(ReportGenerationFailure value) {
		return new GenrapPluginEvent(1, value);
	}


	public static GenrapPluginEvent mkSuccess(
			String xmlFileName
			, String xsdFileName
			, String reportFileName
			, String outputFileName
			, OutputKind outputKind
	) {
		return createSuccess( ReportParams.create(
							xmlFileName
							, xsdFileName
							, reportFileName
							, outputFileName
							, outputKind
							 ));
	}
	
	public static GenrapPluginEvent mkFailure(
			ReportParams params
			, String cause
	) {
		return createFailure( ReportGenerationFailure.create(
							params
							, cause
							 ));
	}
	

	/** @return true, when this is success */
	public boolean isSuccess() {
		return variant == 0;
	}

	/** @return variant value, when this is success; throws Exception otherwise */
	public ReportParams asSuccess() {
		SchemaException.ensure(isSuccess());
		return (ReportParams)value;
	}

	/** @return true, when this is failure */
	public boolean isFailure() {
		return variant == 1;
	}

	/** @return variant value, when this is failure; throws Exception otherwise */
	public ReportGenerationFailure asFailure() {
		SchemaException.ensure(isFailure());
		return (ReportGenerationFailure)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isSuccess()) {
			;
		} else if (isFailure()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isSuccess()) {
			asSuccess();
		} else if (isFailure()) {
			asFailure();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isSuccess())
			return "success" + "(" + asSuccess() + ")";
		if (isFailure())
			return "failure" + "(" + asFailure() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(GenrapPluginEvent obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof GenrapPluginEvent) 
			&& this.eq((GenrapPluginEvent)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for GenrapPluginEvent
	 */
	public static interface MM {
		public void invoke(GenrapPluginEvent cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for GenrapPluginEvent
	 */
	public static interface MMIntf {
		public void success(
				 String xmlFileName
				, String xsdFileName
				, String reportFileName
				, String outputFileName
				, OutputKind outputKind
		);
		public void failure(
				 ReportParams params
				, String cause
		);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for GenrapPluginEvent
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(GenrapPluginEvent cmd) {
				 if (cmd.isSuccess()) {
					ReportParams val = cmd.asSuccess();
					mm.success(
							 val.getXmlFileName()
							, val.getXsdFileName()
							, val.getReportFileName()
							, val.getOutputFileName()
							, val.getOutputKind()
					);
				} else if (cmd.isFailure()) {
					ReportGenerationFailure val = cmd.asFailure();
					mm.failure(
							 val.getParams()
							, val.getCause()
					);
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void success(
							 String xmlFileName
							, String xsdFileName
							, String reportFileName
							, String outputFileName
							, OutputKind outputKind
			) {
				mm.invoke( GenrapPluginEvent.createSuccess(
								ReportParams.create( 
									xmlFileName, xsdFileName, reportFileName, outputFileName, outputKind)));		
			}
		
			public void failure(
							 ReportParams params
							, String cause
			) {
				mm.invoke( GenrapPluginEvent.createFailure(
								ReportGenerationFailure.create( 
									params, cause)));		
			}
		
		}
	}
}