package nianio.ids;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class DialogId = variant {
 *	    fileChooser of FileChooserId  
 *	 |  user of UserDialogId  
 *	 |  message of MessageDialogId  
 * }</pre>
 */ 
public class DialogId implements ImmObject {
	private static final int TYPE_HASH = -169199453;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private DialogId(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static DialogId createFileChooser(FileChooserId value) {
		return new DialogId(0, value);
	}

	public static DialogId createUser(UserDialogId value) {
		return new DialogId(1, value);
	}

	public static DialogId createMessage(MessageDialogId value) {
		return new DialogId(2, value);
	}



	/** @return true, when this is fileChooser */
	public boolean isFileChooser() {
		return variant == 0;
	}

	/** @return variant value, when this is fileChooser; throws Exception otherwise */
	public FileChooserId asFileChooser() {
		SchemaException.ensure(isFileChooser());
		return (FileChooserId)value;
	}

	/** @return true, when this is user */
	public boolean isUser() {
		return variant == 1;
	}

	/** @return variant value, when this is user; throws Exception otherwise */
	public UserDialogId asUser() {
		SchemaException.ensure(isUser());
		return (UserDialogId)value;
	}

	/** @return true, when this is message */
	public boolean isMessage() {
		return variant == 2;
	}

	/** @return variant value, when this is message; throws Exception otherwise */
	public MessageDialogId asMessage() {
		SchemaException.ensure(isMessage());
		return (MessageDialogId)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isFileChooser()) {
			;
		} else if (isUser()) {
			;
		} else if (isMessage()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isFileChooser()) {
			asFileChooser();
		} else if (isUser()) {
			asUser();
		} else if (isMessage()) {
			asMessage();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isFileChooser())
			return "fileChooser" + "(" + asFileChooser() + ")";
		if (isUser())
			return "user" + "(" + asUser() + ")";
		if (isMessage())
			return "message" + "(" + asMessage() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(DialogId obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof DialogId) 
			&& this.eq((DialogId)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	
}