#!/usr/bin/perl

use WarningsFilter;

sub get {
	my ($x) = @_;
	return $x =~ /'(.*?)'/g;
}

sub value {
	my ($value, %x) = @_;
	$result = $x{$value};
	if (!defined $result) {
		die "no $value inside imm";
	}
	return $result;
}

# @param - list of lines, returns: hashmap for a 'dict of' and 'imm'
sub readMsgs {
	my ($linesRef) = @_;
	my @lines = @$linesRef;
	#print "size = $#lines\n";
	my %dicts = ();
	my %imms = ();
	foreach my $line (@lines) {
		$line=~s/'msg'={(.*?)}/$1/;
		#print "line = $line";
		%msg = &get($line);
		my $file = value("filePath", %msg);
		my $msgType = value("type", %msg);
		if ($msgType =~ "dictOf") {
			my $typeRef = value("dictOf", %msg);
			$dicts{$file."\\".$typeRef} = {filePath=>$file, typeRef=>$typeRef, errType=>$msgType};
		} elsif ($msgType =~ "immUsage") {
			my $typeRef = value("immUsage", %msg);
			$imms{$file."\\".$typeRef} = {filePath=>$file, typeRef=>$typeRef, errType=>$msgType};
		} else {
			die "unknown variant " . $msgType;
		}
		
	}
	return (\%dicts, \%imms);
}

sub asLines {
	my ($msgsFile) = @_;
	open IN, "<$msgsFile" or die "Can't open file $msgsFile";
	my @lines = <IN>;
	close IN;
	return \@lines;
}

# true if there are elements in param tested that are not inside param lib. Prints these elements.
sub includesDiff {
	my ($libRef, $testedRef) = @_;
	my %lib = %$libRef;
	my %tested = %$testedRef;
	my @different;
	foreach (keys %tested) {
		if (!defined $lib{$_}) {
			$valueRef = $tested{$_};
			&printErr($valueRef);
			push(@different, %$valueRef);
		}
	}
	return (@different > 0);
}

sub printErr {
	my ($valueRef) = @_;
	my %value = %$valueRef;
	print "Error in file ".$value{"filePath"} .": \n  ";
	my $errType = $value{"errType"};
	my $typeRef = $value{"typeRef"};
	if ($errType =~ "dictOf") {
		print "Deprecated construction '$typeRef = dict of ...'\n";	
	} elsif ($errType =~ "immUsage") {
		print "Type $typeRef uses deprecated imm type '[ java imm ] primitive'\n";	
	} else {
		die "unknown variant " . $errType;
	}
}

sub printUsage {
	print "USAGE: $0 libraryFile [ignoreErrors] < messagesFile\n";
	print "If ignoreErrors is given than errors will not be checked.";
}

sub check {
	my $libFile = $ARGV[0] or die printUsage();
	my $ignoreErrors = $ARGV[1];
	my ($warnLinesRef) = &WarningsFilter::filterInputWarnings;
	if (!$ignoreErrors) {
		my ($inDicts, $inImms) = &readMsgs($warnLinesRef);
		my ($libDicts, $libImms) = &readMsgs(&asLines($libFile));
		my ($dictOfErr) = &includesDiff($libDicts, $inDicts);
		my ($immErr) = &includesDiff($libImms, $inImms);
		if ($dictOfErr or $immErr) {
			die "";
		}
	}
}

&check;
